SystemOrganization addCategory: #ConfigurationOfBootstrap!

Object subclass: #ConfigurationOfBootstrap
	instanceVariableNames: 'project'
	classVariableNames: 'LastVersionLoad'
	poolDictionaries: ''
	category: 'ConfigurationOfBootstrap'!

!ConfigurationOfBootstrap commentStamp: '<historical>' prior: 0!
#The Bootstrap for Seaside project

## Introduction
### The project
The **[Bootstrap for Seaside][1]** helps you to easily use the [Bootstrap][2] library within your [Seaside][3] web application.

### Project repository

Project repository can be found at [http://smalltalkhub.com/#!!/~TorstenBergmann/Bootstrap][4]

There you will also find the documentation and install instructions.

#### Why should you use it
[Bootstrap][5] is a front-end toolkit for rapidly developing web applications. It is a collection of CSS and HTML conventions and design templates for typography, forms, buttons, navigation and other interface components. It helps to enforce a common and professional looking style for your own web application.

[Seaside][6] is one of the most powerful web framework ever invented - with its heretic approach it allows for reusability and by using the underlying language and tools it makes you very productive.

By combining the two frameworks using the **[Bootstrap for Seaside][7]** project (or "Bootstrap" for short) the advantages are on your side:

 - As a ***Seaside developer*** you can continue with your power framework and easily give your components a modern and fresh look with Bootstrap which can also be styled independently by a designer afterwards. 

 - As a ***Bootstrap user*** you can use the power of Seaside to write reusable components and combine them to full working applications easily.

#### Did'nt we have similar projects in the past already

There were already two projects for Bootstrap and Seaside provided in the past:

  1. [http://ss3.gemstone.com/ss/TwitterBootstrap.html][8]
  2. [http://smalltalkhub.com/#!!/~GastonDallOglio/TBootstrap][9]
  
Both can still be used but are outdated. They dont have a full test suite, provide no examples and do not rely on the latest Bootstrap version. So we would recommend to use the  **[Bootstrap for Seaside][10]** project since it comes with more than 160 tests and is up to date.

#### Migration

If you use one of the two before mentioned projects you can easily migrate your code. While project (1) uses the "TWBS" prefix and (2) the "TB" prefix for classes and selectors we use "TBS" as the prefix for classes and method selectors.

## Installation

### Installation using config browser

You can install the project from the Pharo configuration browser. [Just install and open a fresh Pharo 2.0 image][11], click on the desktop and select *"Tools"* -> *"Configuration Browser"* from the world menu. Select "Bootstrap" and click on install.

Note that by default it loads the Seaside3 framework as a dependent project - so if you started from a fresh and clean Pharo image you end up in a full Seaside development environment with the project on top of it.

### Installation using Gofer

You can also install the project using the built in *Gofer* class that allows you to automate the loading. Just open a workspace and evaluate:

    Gofer new
        url: 'http://smalltalkhub.com/mc/TorstenBergmann/Bootstrap/main';
        package: 'ConfigurationOfBootstrap';
        load.

    (Smalltalk at: #ConfigurationOfBootstrap) project stableVersion load

### Running the examples

The project comes with a set of examples that you can open in the webbrowser. First start the webserver:

     ZnZincServerAdaptor startOn: 8080
     
and then browse the examples at the following location: http://localhost:8080/bootstrap

As you will see you will find nearly all Bootstrap components:

 - Well
 - Jumbotron
 - Panels
 - Alarms
 - Buttons
 - Breadcrumbs
 - Navs and Navbars
 - Tables
 - Pagination
 - ...

The examples are oriented towards the HTML examples given on the [Bootstrap website][12] and also show the according Smalltalk code. So you can easily see how to use the component set in your own projects. 

## Architecture 

### Package structure and naming conventions

The project contains three basic packages:

 - Bootstrap-Core
 - Bootstrap-Tests
 - Bootstrap-Examples

The classes use a "TBS" prefix and the method extensions the "tbs" prefix. 

### Using the project in own applications

If you want to use the project in your own web project only the "Bootstrap-Core" package is required. 

#### Registering the file libraries
To use the project just add the bootstrap file library to your Seaside application, depending on state of your project use either the ***TBSDevelopmentLibrary*** (for development) or ***TBSDeploymentLibrary*** (for production). Also note that the JQuery library is required.

    |app|
	app := WAAdmin register: self asApplicationAt: 'mykillerwebapp'.
	app 
		addLibrary: JQDeploymentLibrary;
		addLibrary: TBSDeploymentLibrary

#### Write your code

You can now use the additional methods in your usual ***renderContentOn:*** methods for Seaside. Here is an example:

    renderContentOn: html
        html tbsContainer: [
            html text: 'A text within a bootstrap container']
            
Have a look at the various examples - they will show you how to use the components.

#### Tips & Tricks
Bootstrap has an easy and powerful mechanism to work with layouts. Have a look at the "Grid" examples that come with the project.

By default the Bootstrap positioned rows/columns are invisible. To easily find out if the layout is correct just use the message #showGrid: for a #tbsColumn. This will make the layout visible by adding a background color and border - so you can control the placement on the screen and resizing behavior. Note that this only works with the ***TBSDevelopmentLibrary*** file library, so it is not activated in production.

### Code Contributions

The project is MIT licensed and accepts contributions. If you want to contribute just check that your contributed code is covered by automatic tests and keeps the projects test suite green.

Also note that the Bootstrap library itself is under the Apache 2.0 license and adds a "*Copyright 2013 Twitter, Inc*".

### CI

The project is controlled by a CI server under: [https://ci.inria.fr/pharo-contribution/job/Bootstrap/][13]

## Thanks 

**Thanks go to** 

 - the Bootstrap team who invented the Bootstrap framework at Twitter (Mark Otto, Jacob Thornton and contributors)
 - the contributors of the first Seaside based [TwitterBootstrap][14] project (Nick Ager, Jan van de Sandt, Gastn Dall' Oglio, Tobias Pape, Tudor Girba, Paul DeBruicker, Francois Stephany, Philippe Marschall, Norbert Hartl, Yanni Chiu, Diego Lont)
 - to Gaston Dall Oglio (creator of [TBootstrap][15]) 


  [1]: http://smalltalkhub.com/#!!/~TorstenBergmann/Bootstrap
  [2]: http://getbootstrap.com/
  [3]: http://www.seaside.st/
  [4]: http://smalltalkhub.com/#!!/~TorstenBergmann/Bootstrap
  [5]: http://getbootstrap.com/
  [6]: http://www.seaside.st/
  [7]: http://smalltalkhub.com/#!!/~TorstenBergmann/Bootstrap
  [8]: http://ss3.gemstone.com/ss/TwitterBootstrap.html
  [9]: http://smalltalkhub.com/#!!/~GastonDallOglio/TBootstrap
  [10]: http://smalltalkhub.com/#!!/~TorstenBergmann/Bootstrap
  [11]: http://files.pharo.org/platform
  [12]: http://getbootstrap.com
  [13]: https://ci.inria.fr/pharo-contribution/job/Bootstrap/
  [14]: http://ss3.gemstone.com/ss/TwitterBootstrap.html
  [15]: http://smalltalkhub.com/#!!/~GastonDallOglio/TBootstrap!

!ConfigurationOfBootstrap class methodsFor: 'development support' stamp: 'TorstenBergmann 10/1/2013 17:32'!
DevelopmentSupport

"See the methods in the 'development support' category on the class-side of MetacelloBaseConfiguration. Decide what development support methods you would like to use and copy them the the class-side of your configuration."
	<apiDocumentation>
! !

!ConfigurationOfBootstrap class methodsFor: 'private' stamp: 'TorstenBergmann 10/1/2013 17:32'!
baseConfigurationClassIfAbsent: aBlock

	^Smalltalk
		at: #MetacelloBaseConfiguration
		ifAbsent: [ 
			self ensureMetacelloBaseConfiguration.
			Smalltalk at: #MetacelloBaseConfiguration ifAbsent: aBlock ].
! !

!ConfigurationOfBootstrap class methodsFor: 'documentation support' stamp: 'TorstenBergmann 10/2/2013 23:58'!
bootstrapTutorialOn: aBuilder
	<onlineTutorial>
	
	(aBuilder tutorial: #'Bootstrap')		 
		description: 'Using Bootstrap and Seaside within Pharo.';
		addLesson:  ((aBuilder lesson: #'The Bootstrap project')						 
						description: 'A short overview of the Bootstrap project.';
						addPage: #documentation from: self)! !

!ConfigurationOfBootstrap class methodsFor: 'catalog' stamp: 'TorstenBergmann 10/3/2013 00:00'!
catalogDescription

^ 
'"Bootstrap for Seaside" is a project combining Seaside and Twitter Bootstrap and easily use them to write professional looking web applications.'! !

!ConfigurationOfBootstrap class methodsFor: 'catalog' stamp: 'TorstenBergmann 10/3/2013 00:01'!
catalogKeywords
	^ #(HTML web query Seaside Bootstrap)
	! !

!ConfigurationOfBootstrap class methodsFor: 'documentation support' stamp: 'TorstenBergmann 10/2/2013 23:57'!
documentation
	"The docu was written using http://benweet.github.io/stackedit/
	  and stored as class comment.
	"
	
	^self comment! !

!ConfigurationOfBootstrap class methodsFor: 'private' stamp: 'TorstenBergmann 10/1/2013 17:32'!
ensureMetacello

	(self baseConfigurationClassIfAbsent: []) ensureMetacello! !

!ConfigurationOfBootstrap class methodsFor: 'private' stamp: 'TorstenBergmann 10/1/2013 17:32'!
ensureMetacelloBaseConfiguration

	Smalltalk
		at: #MetacelloBaseConfiguration
		ifAbsent: [ 
			| repository version |
			repository := MCHttpRepository location: 'http://seaside.gemstone.com/ss/metacello' user: '' password: ''.
			repository
				versionReaderForFileNamed: 'Metacello-Base-DaleHenrichs.2.mcz'
				do: [ :reader | 
					version := reader version.
					version load.
					version workingCopy repositoryGroup addRepository: repository ] ]! !

!ConfigurationOfBootstrap class methodsFor: 'metacello tool support' stamp: 'TorstenBergmann 10/1/2013 17:32'!
isMetacelloConfig
	"Answer true and the Metacello tools will operate on you"
	
	^true! !

!ConfigurationOfBootstrap class methodsFor: 'loading' stamp: 'TorstenBergmann 10/1/2013 17:32'!
load
	"Load the #stable version defined for this platform. The #stable version is the version that is recommended to be used on this platform."

	"self load"

	<apiDocumentation>
	^(self project version: #stable) load! !

!ConfigurationOfBootstrap class methodsFor: 'loading' stamp: 'TorstenBergmann 10/1/2013 17:32'!
loadBleedingEdge
	"Load the latest versions of the mcz files defined for this project. It is not likely that the #bleedingEdge has been tested."

	"self loadBleedingEdge"

	<apiDocumentation>
	^(self project version: #bleedingEdge) load! !

!ConfigurationOfBootstrap class methodsFor: 'loading' stamp: 'TorstenBergmann 10/1/2013 17:32'!
loadDevelopment
	"Load the #development version defined for this platform. The #development version will change over time and is not expected to be stable."

	"self loadDevelopment"

	<apiDocumentation>
	^(self project version: #development) load! !

!ConfigurationOfBootstrap class methodsFor: 'accessing' stamp: 'TorstenBergmann 10/1/2013 17:32'!
project

	^self new project! !

!ConfigurationOfBootstrap class methodsFor: 'development support' stamp: 'TorstenBergmann 10/1/2013 17:32'!
validate
	"Check the configuration for Errors, Critical Warnings, and Warnings (see class comment for MetacelloMCVersionValidator for more information). 
	Errors identify specification issues that will result in unexpected behaviour when you load the configuration. 
	Critical Warnings identify specification issues that may result in unexpected behavior when you load the configuration.
	Warnings identify specification issues that are technically correct, but are worth take a look at."

	"self validate"

	<apiDocumentation>
	self ensureMetacello.
	^ ((Smalltalk at: #MetacelloToolBox) validateConfiguration: self debug: #() recurse: false) explore! !

!ConfigurationOfBootstrap methodsFor: 'baselines' stamp: 'TorstenBergmann 10/2/2013 21:06'!
baseline010: spec 
        <version: '0.1.0-baseline'>
        
	spec for: #common do: [
		spec blessing: #baseline.
		spec description: 'First baseline'.
            spec author: 'Torsten Bergmann'.
            spec timestamp: '10/2/2012 20:00'.		
		spec repository: 'http://smalltalkhub.com/mc/TorstenBergmann/Bootstrap/main/'.    
		
		spec project: 'Seaside3' with: [                                                     
                                spec
                                        className: 'ConfigurationOfSeaside3';
                                        repository: 'http://www.smalltalkhub.com/mc/Seaside/MetacelloConfigurations/main' ].
				            
		spec package: 'Bootstrap-Core' with: [ spec requires: 'Seaside3'].
		spec package: 'Bootstrap-Tests-Core'  with: [ spec requires: 'Bootstrap-Core' ].
		spec package: 'Bootstrap-Examples'  with: [ spec requires: 'Bootstrap-Core'].
		
		spec group: 'default' 
                     	with:   #('Bootstrap-Core' 'Bootstrap-Tests-Core' 'Bootstrap-Examples')].! !

!ConfigurationOfBootstrap methodsFor: 'accessing' stamp: 'TorstenBergmann 10/1/2013 17:32'!
customProjectAttributes
	 "Edit to return a collection of any custom attributes e.g. for conditional loading: Array with: #'Condition1' with: #'Condition2.
	For more information see: http://code.google.com/p/metacello/wiki/CustomProjectAttrributes"

	^ #().! !

!ConfigurationOfBootstrap methodsFor: 'symbolic versions' stamp: 'TorstenBergmann 10/2/2013 20:51'!
development: spec
	<symbolicVersion: #'development'>

	spec for: #'common' version: '0.1.0'.
! !

!ConfigurationOfBootstrap methodsFor: 'accessing' stamp: 'TorstenBergmann 10/1/2013 17:32'!
project

	^ project ifNil: [ | constructor |
		"Bootstrap Metacello if it is not already loaded"
		(self class baseConfigurationClassIfAbsent: []) ensureMetacello.
		"Construct Metacello project"
		project := MetacelloMCProject new projectAttributes: self customProjectAttributes. 
		constructor := (Smalltalk at: #MetacelloVersionConstructor) on: self project: project.
		project loadType: #linear. "change to #atomic if desired"
		project ]! !

!ConfigurationOfBootstrap methodsFor: 'symbolic versions' stamp: 'TorstenBergmann 10/2/2013 20:51'!
stable: spec
	<symbolicVersion: #'stable'>

	spec for: #'common' version: '0.1.0'.
! !

!ConfigurationOfBootstrap methodsFor: 'versions' stamp: 'TorstenBergmann 10/2/2013 20:59'!
version010: spec 
        <version: '0.1.0' imports: #('0.1.0-baseline')>

        spec for: #common do: [
	            spec blessing: #development.

			spec project: 'Seaside3' with: '3.0.8'.			
			spec package: 'Bootstrap-Core' with: 'Bootstrap-Core-TorstenBergmann.18'.
			spec package: 'Bootstrap-Tests-Core' with: 'Bootstrap-Tests-Core-TorstenBergmann.14'.
			spec package: 'Bootstrap-Examples' with: 'Bootstrap-Examples-TorstenBergmann.18'.
                ].! !
